﻿using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

/// <summary>
/// Редактор
/// </summary>
public class EditorController : MonoBehaviour
{
    public InputField Name;
    public InputField Image;
    public InputField SizeX;
    public InputField SizeY;
    public GameObject BeaconsContent;
    public GameObject PanelBeaconEdit;
    public GameObject Player;
    public GameObject ButtonMode;
    public GameObject EditorTools;
    public Toggle ToggleGrid;
    public Toggle ToggleScalePanel;
    public Toggle projection;
    public GameObject Grid;
    public Dropdown DropdownLocation;
    public Transform Locations;
    public Button ButtonAddBeacon;
    public GameObject Dialog;

    public static GameObject NewLocation;
    public static GameObject plane;

    public List<Beacon> Beacons = new List<Beacon>();

    PlayerController player;
    ModeController mode;
    ToolsController tools;

    // Start is called before the first frame update
    void Start()
    {
        player = PlayerController.instance;
        mode = ButtonMode.GetComponent<ModeController>();
        tools = EditorTools.GetComponent<ToolsController>();

        ButtonAddBeacon.onClick.AddListener(() => {
            AddBeacon(player);
        });
    }

    // Update is called once per frame
    void Update()
    {
        if (NewLocation == null)
        {
            NewLocation = new GameObject();
            NewLocation.transform.SetParent(Locations);
            NewLocation.name = "NewLocation";
            NewLocation.transform.position = Vector3.zero;
            NewLocation.SetActive(false);
            plane = Instantiate(Resources.Load("GameObjects/Plane", typeof(GameObject))) as GameObject;
            plane.transform.SetParent(NewLocation.transform);
            List<string> name = new List<string> {"Walls", "Zones", "Beacons" };
            for (int i = 0; i < 3; i++)
            {
                var go = new GameObject();
                go.name = name[i];
                go.transform.SetParent(NewLocation.transform);
            }
            var location = new Location { 
                location = NewLocation,
                walls = new List<Wall>(),
                zones = new List<Zone>(),
                beacons = new List<Beacon>() };
            player.locations[location.id] = location;
            player.locations_index.Add(location.id);
        }

        SizeX.text = $"{plane.transform.localScale.x}";
        SizeY.text = $"{plane.transform.localScale.z}";
        if (ToggleGrid.isOn != Grid.activeSelf) Grid.SetActive(ToggleGrid.isOn);
        if (ToggleScalePanel.isOn != plane.GetComponent<TouchScript>().activeScript) plane.GetComponent<TouchScript>().activeScript = ToggleScalePanel.isOn;

        if (projection.isOn)
        {
            Camera.main.orthographic = true;
        }
        else
        {
            Camera.main.orthographic = false;
        }


        //PlayerController.LoadMaps(DropdownLocation, Locations.gameObject);
    }

    public void UpdatePanel()
    {
        if (!Image.text.Equals(""))
            StartCoroutine(SendingFormController.LoadImage(Image.text, plane));

        var x = 1f;
        if (!SizeX.text.Equals(""))
            x = float.Parse(SizeX.text, CultureInfo.InvariantCulture);
        var y = 1f;
        if (!SizeY.text.Equals(""))
            y = float.Parse(SizeY.text, CultureInfo.InvariantCulture);
        plane.transform.localScale = new Vector3(x, 1, y); 
        var scale = plane.transform.localScale;
        plane.transform.position = new Vector3(scale.x * 5, 0, scale.z * 5);
    }

    public void Save()
    {
        //SceneManager.LoadScene("Location");
        tools.CursorOff();      
        if (!Name.text.Equals("")) {

            //var location = new Location { id = 0, name = Name.text, beacons = Beacons, zones = WallCreate.Zones, walls = WallCreate.Walls, texture_url = Image.text, plane = plane, location =  NewLocation};
            var location = player.locations[player.locations_index[player.active_location]];
            location.name = Name.text;
            location.texture_url = Image.text;
            location.walls.AddRange(WallCreate.Walls);
            location.zones.AddRange(WallCreate.Zones);
            location.beacons.AddRange(Beacons);
            
            player.DropdownLocation.options.Last().text = location.name;

            location.location.name = location.name;
            //location.location.transform.SetParent(Locations);
            //foreach(var b in location.beacons)
            //    b.beacon.transform.SetParent(location.location.transform);
            //foreach (var w in location.walls)
            //    w.go.transform.SetParent(location.location.transform);
            //foreach (var z in location.zones)
            //    z.go.transform.SetParent(location.location.transform);

            location.Save();
            mode.Switch();
        }
        else {
            Dialog.SetActive(true);
            Dialog.transform.GetChild(0).GetComponent<Text>().text = "Введите название локации";
            Dialog.transform.GetChild(2).GetComponent<Button>().onClick.AddListener(() => {
                Dialog.SetActive(false);
            });
        }
    }

    public void Back()
    {
        //SceneManager.LoadScene("Location");
        mode.Switch();
    }

    public void AddBeacon(PlayerController player, Beacon b = null)
    {
        //PanelBeaconEdit.SetActive(true);
        var location = player.locations[player.locations_index[player.active_location]].location;

        var beacon = Instantiate(Resources.Load("GameObjects/Beacon", typeof(GameObject))) as GameObject;
        if(ModeController.editor) beacon.GetComponent<BeaconController>().mode = BeaconController.Mode.Editor;
        
        beacon.name = $"Beacon_{b?.id.ToString() ?? Beacons.Count.ToString()}";
        if (b != null)
        {           
            beacon.transform.position = new Vector3(b.x / 100, 0.5f, b.y / 100);
            //beacon.transform.GetChild(0).GetChild(0).GetChild(0).GetComponent<Text>().text 
            beacon.GetComponent<BeaconController>().text_info= $"{b.id}\n{b.vagrant_label}\nuuid={b.uuid}\nmajor={b.major}\nminor={b.minor}\nmac={b.mac}";
            beacon.GetComponent<BeaconController>().id = $"{b.id}";
            if (b.enabled == false) beacon.GetComponent<Renderer>().material.color = Color.grey;
        }
        beacon.transform.SetParent(location.transform.Find("Beacons").transform);          
        var name_beacon = $"Маяк {b?.id.ToString() ?? Beacons.Count.ToString()}";
        PanelBeaconEdit.transform.GetChild(0).GetComponent<Text>().text = name_beacon;
        var panel_button = Instantiate(Resources.Load("GameObjects/Panel_Beacon", typeof(GameObject))) as GameObject;
        panel_button.transform.SetParent(BeaconsContent.transform);
        panel_button.name = $"Beacon_{b?.id.ToString() ?? Beacons.Count.ToString()}";
        var button = panel_button.transform.GetChild(0).GetComponent<Button>();      
        button.name = $"BeaconButton_{Beacons.Count}";
        
        button.transform.GetChild(0).GetComponent<Text>().text = name_beacon;

        //var b = new Beacon { };

        button.onClick.AddListener(() =>
        {
            PanelBeaconEdit.SetActive(true);
            PanelBeaconEdit.transform.GetChild(0).GetComponent<Text>().text = name_beacon;
            if (b != null)
            {
                PanelBeaconEdit.transform.GetChild(1).GetComponent<InputField>().text = b.uuid;
                PanelBeaconEdit.transform.GetChild(1).GetComponent<InputField>().text = b.vagrant_label.ToString();
                PanelBeaconEdit.transform.GetChild(3).GetComponent<InputField>().text = $"{b.major}";
                PanelBeaconEdit.transform.GetChild(4).GetComponent<InputField>().text = $"{b.major}";
                PanelBeaconEdit.transform.GetChild(5).GetComponent<InputField>().text = b.mac;
            }
        });
        //var text = PanelBeaconEdit.transform.GetChild(0).GetComponent<Text>().text = "";
        //var uuid = PanelBeaconEdit.transform.GetChild(1).GetComponent<InputField>().text = "";
        if (b == null) b = new Beacon();
        var button_ok = PanelBeaconEdit.transform.GetChild(7).transform.GetChild(0).GetComponent<Button>();
        button_ok.onClick.AddListener(() =>
        {
            b.uuid = PanelBeaconEdit.transform.GetChild(1).GetComponent<InputField>().text;
            b.minor = ushort.Parse(PanelBeaconEdit.transform.GetChild(2).GetComponent<InputField>().text);
            b.major = ushort.Parse(PanelBeaconEdit.transform.GetChild(3).GetComponent<InputField>().text);

            PanelBeaconEdit.SetActive(false);
        });
        var button_close = PanelBeaconEdit.transform.GetChild(7).transform.GetChild(1).GetComponent<Button>();
        button_close.onClick.AddListener(() =>
        {
            PanelBeaconEdit.SetActive(false);
        });

         var button_del = panel_button.transform.GetChild(2).GetComponent<Button>();
        button_del.onClick.AddListener(() =>
        {
            Destroy(beacon);
            Destroy(panel_button);
            Beacons.Remove(b);
            if (PanelBeaconEdit.activeSelf) PanelBeaconEdit.SetActive(false);
        });

        //b.id = (uint)Beacons.Count;
        b.beacon = beacon;
        b.panel_button = panel_button;

        Beacons.Add(b);
    }
}
